package com.diagbot.entity;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.diagbot.util.StringUtil;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 诊断映射表
 * </p>
 *
 * @author zhaops
 * @since 2020-07-28
 */
@TableName("tran_disease_config")
public class DiseaseConfig implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 是否删除,N:未删除，Y:删除
     */
    private String isDeleted;

    /**
     * 记录创建时间
     */
    private Date gmtCreate;

    /**
     * 记录修改时间,如果时间是1970年则表示纪录未修改
     */
    private Date gmtModified;

    /**
     * 创建人，0表示无创建人值
     */
    private String creator;

    /**
     * 修改人,如果为0则表示纪录未修改
     */
    private String modifier;

    /**
     * 医院id
     */
    private Long hospitalId;

    /**
     * 医院诊断名称
     */
    @Excel(name = "医院诊断名称", width = 40, orderNum = "1", isImportField = "true")
    @NotBlank(message = "请输入医院诊断名称")
    private String hisName;

    /**
     * 标准诊断名称
     */
    @Excel(name = "标准诊断名称", width = 40, orderNum = "3", isImportField = "true")
    @NotBlank(message = "请输入标准诊断名称")
    private String uniqueName;

    /**
     * ICD-10编码
     */
    @Excel(name = "ICD-10编码", width = 40, orderNum = "2", isImportField = "true")
    private String icdCode;

    /**
     * 是否匹配（0-未匹配、1-已匹配）
     */
    @Excel(name = "是否匹配", width = 20, orderNum = "4", replace = {"未匹配_0,已匹配_1,_null"},isImportField = "true")
    private Integer isMatch;

    /**
     * 数据来源（1-标准词、2-同义词、3-编码、4-历史数据、5-相似词、99-数据迁移）
     */
    @Excel(name = "数据来源", width = 20, orderNum = "5", replace = {"标准词_1,同义词_2,编码_3,历史数据_4,相似词_5,数据迁移_99,_null"},isImportField = "true")
    private Integer source;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getIsDeleted() {
        return isDeleted;
    }

    public void setIsDeleted(String isDeleted) {
        this.isDeleted = isDeleted;
    }

    public Date getGmtCreate() {
        return gmtCreate;
    }

    public void setGmtCreate(Date gmtCreate) {
        this.gmtCreate = gmtCreate;
    }

    public Date getGmtModified() {
        return gmtModified;
    }

    public void setGmtModified(Date gmtModified) {
        this.gmtModified = gmtModified;
    }

    public String getCreator() {
        return creator;
    }

    public void setCreator(String creator) {
        this.creator = creator;
    }

    public String getModifier() {
        return modifier;
    }

    public void setModifier(String modifier) {
        this.modifier = modifier;
    }

    public Long getHospitalId() {
        return hospitalId;
    }

    public void setHospitalId(Long hospitalId) {
        this.hospitalId = hospitalId;
    }

    public String getHisName() {
        return hisName;
    }

    public void setHisName(String hisName) {
        this.hisName = hisName;
    }

    public String getUniqueName() {
        return uniqueName;
    }

    public void setUniqueName(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    public String getIcdCode() {
        return icdCode;
    }

    public void setIcdCode(String icdCode) {
        this.icdCode = icdCode;
    }

    public Integer getIsMatch() {
        return isMatch;
    }

    public void setIsMatch(Integer isMatch) {
        this.isMatch = isMatch;
    }

    public Integer getSource() {
        return source;
    }

    public void setSource(Integer source) {
        this.source = source;
    }

    @Override
    public String toString() {
        return "DiseaseConfig{" +
                "id=" + id +
                ", isDeleted=" + isDeleted +
                ", gmtCreate=" + gmtCreate +
                ", gmtModified=" + gmtModified +
                ", creator=" + creator +
                ", modifier=" + modifier +
                ", hospitalId=" + hospitalId +
                ", hisName=" + hisName +
                ", uniqueName=" + uniqueName +
                ", icdCode=" + icdCode +
                ", isMatch=" + isMatch +
                ", source=" + source +
                "}";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        DiseaseConfig diseaseConfig = (DiseaseConfig) o;
        return Objects.equals(id, diseaseConfig.id)
                && Objects.equals(isDeleted, diseaseConfig.isDeleted)
                && Objects.equals(hospitalId, diseaseConfig.hospitalId)
                && Objects.equals(hisName, diseaseConfig.hisName)
                && Objects.equals(uniqueName, diseaseConfig.uniqueName)
                && Objects.equals(icdCode, diseaseConfig.icdCode);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, isDeleted, hospitalId, hisName, uniqueName, icdCode);
    }

    public static boolean nonNull(DiseaseConfig o) {
        return !(o == null
                || (o.hospitalId == null
                && StringUtil.isBlank(o.hisName)
                && StringUtil.isBlank(o.uniqueName)
                && StringUtil.isBlank(o.icdCode)));
    }
}