package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.AnesthesiaConfig;
import com.diagbot.entity.DeptConfig;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.DrugConfig;
import com.diagbot.entity.LisConfig;
import com.diagbot.entity.MappingConfig;
import com.diagbot.entity.NurseConfig;
import com.diagbot.entity.OperationConfig;
import com.diagbot.entity.PacsConfig;
import com.diagbot.entity.ScaleConfig;
import com.diagbot.entity.TcmdiseaseConfig;
import com.diagbot.entity.TcmsyndromeConfig;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.entity.wrapper.MappingConfigWrapper;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.MatchSourceEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.MappingConfigServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.MappingConfigPageVO;
import com.google.common.collect.Lists;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/6/9 14:19
 */
@Component
public class MappingConfigFacade extends MappingConfigServiceImpl {
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 分页查询
     *
     * @param mappingConfigPageVO
     * @return
     */
    @Override
    public IPage<MappingConfigWrapper> getPage(@Param("mappingConfigPageVO") MappingConfigPageVO mappingConfigPageVO) {
        if (mappingConfigPageVO.getHospitalId() == null) {
            mappingConfigPageVO.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }

        FilterVO filterVO = new FilterVO();
        RespDTO<List<IndexBatchDTO>> respDTO = null;

        //todo 标准词转conceptId
        if (StringUtil.isNotBlank(mappingConfigPageVO.getUniqueName())) {
            filterVO.setInputStr(mappingConfigPageVO.getUniqueName());
            respDTO = cdssCoreClient.filter(filterVO);
            if (RespDTOUtil.respIsOK(respDTO) && ListUtil.isNotEmpty(respDTO.data)) {
                mappingConfigPageVO.setConceptIds(respDTO.data
                        .stream()
                        .map(IndexBatchDTO::getId)
                        .distinct()
                        .collect(Collectors.toList()));
            }
        }

        //todo 药品剂型转 conceptId
        if (StringUtil.isNotBlank(mappingConfigPageVO.getForm())) {
            filterVO.setInputStr(mappingConfigPageVO.getForm());
            respDTO = cdssCoreClient.filter(filterVO);
            if (RespDTOUtil.respIsOK(respDTO) && ListUtil.isNotEmpty(respDTO.data)) {
                mappingConfigPageVO.setFormConceptIds(respDTO.data
                        .stream()
                        .filter(i -> i.getType().equals(ConceptTypeEnum.Form.getKey()))
                        .map(IndexBatchDTO::getId)
                        .distinct()
                        .collect(Collectors.toList()));
            }
        }

        //todo 多条匹配
        if (mappingConfigPageVO.getIsMatch() != null && mappingConfigPageVO.getIsMatch().equals(2)) {
            mappingConfigPageVO.setIsMatch(null);
            mappingConfigPageVO.setIsMultiple(1);
        }

        IPage<MappingConfigWrapper> page = super.getPage(mappingConfigPageVO);

        //todo conceptId转标准词
        List<MappingConfigWrapper> records = page.getRecords();
        ConceptVO conceptVO = new ConceptVO();
        List<Long> ids = Lists.newArrayList();
        ids.addAll(records.stream().map(MappingConfigWrapper::getConceptId).collect(Collectors.toList()));
        ids.addAll(records.stream().filter(i -> i.getFormConceptId() != null)
                .map(MappingConfigWrapper::getFormConceptId).collect(Collectors.toList()));
        conceptVO.setIds(ids);
        RespDTO<List<IndexBatchDTO>> resultRespDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (RespDTOUtil.respIsOK(resultRespDTO) && ListUtil.isNotEmpty(resultRespDTO.data)) {
            Map<Long, String> conceptMap
                    = resultRespDTO.data.stream().collect(Collectors.toMap(IndexBatchDTO::getId, IndexBatchDTO::getName));
            for (MappingConfigWrapper record : records) {
                if (conceptMap.containsKey(record.getConceptId())) {
                    record.setUniqueName(conceptMap.get(record.getConceptId()));
                }
                if (record.getFormConceptId() != null && conceptMap.containsKey(record.getFormConceptId())) {
                    record.setForm(conceptMap.get(record.getFormConceptId()));
                }
            }
            page.setRecords(records);
        }

        return page;
    }

    /**
     * 导入模板下载
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response, Integer type) {
        if (type == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入数据类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉");
        }
        exportExcel(response, new ArrayList<>(), type, "导入模板");
    }

    /**
     * 导出文件
     *
     * @param response
     * @param type
     * @param extFileName
     */
    public void exportExcel(HttpServletResponse response, List<?> list, Integer type, String extFileName) {
        String fileName = extFileName + ".xls";
        switch (type) {
            case 1:
                fileName = "检验" + fileName;
                List<LisConfig> lisConfigList = BeanUtil.listCopyTo(list, LisConfig.class);
                ExcelUtils.exportExcel(lisConfigList, null, "sheet1", LisConfig.class, fileName, response, 12.8f);
                break;
            case 3:
                fileName = "检查" + fileName;
                List<PacsConfig> pacsConfigList = BeanUtil.listCopyTo(list, PacsConfig.class);
                ExcelUtils.exportExcel(pacsConfigList, null, "sheet1", PacsConfig.class, fileName, response, 12.8f);
                break;
            case 4:
                fileName = "诊断" + fileName;
                List<DiseaseConfig> diseaseConfigList = BeanUtil.listCopyTo(list, DiseaseConfig.class);
                ExcelUtils.exportExcel(diseaseConfigList, null, "sheet1", DiseaseConfig.class, fileName, response, 12.8f);
                break;
            case 5:
                fileName = "药品" + fileName;
                List<DrugConfig> drugConfigList = BeanUtil.listCopyTo(list, DrugConfig.class);
                ExcelUtils.exportExcel(drugConfigList, null, "sheet1", DrugConfig.class, fileName, response, 12.8f);
                break;
            case 6:
                fileName = "手术和操作" + fileName;
                List<OperationConfig> operationConfigList = BeanUtil.listCopyTo(list, OperationConfig.class);
                ExcelUtils.exportExcel(operationConfigList, null, "sheet1", OperationConfig.class, fileName, response, 12.8f);
                break;
            case 7:
                fileName = "科室" + fileName;
                List<DeptConfig> deptConfigList = BeanUtil.listCopyTo(list, DeptConfig.class);
                ExcelUtils.exportExcel(deptConfigList, null, "sheet1", DeptConfig.class, fileName, response, 12.8f);
                break;
            case 8:
                fileName = "输血" + fileName;
                List<TransfusionConfig> transfusionConfigList = BeanUtil.listCopyTo(list, TransfusionConfig.class);
                ExcelUtils.exportExcel(transfusionConfigList, null, "sheet1", TransfusionConfig.class, fileName, response, 12.8f);
                break;
            case 10:
                fileName = "量表" + fileName;
                List<ScaleConfig> scaleConfigList = BeanUtil.listCopyTo(list, ScaleConfig.class);
                ExcelUtils.exportExcel(scaleConfigList, null, "sheet1", ScaleConfig.class, fileName, response, 12.8f);
                break;
            case 11:
                fileName = "护理" + fileName;
                List<NurseConfig> nurseConfigList = BeanUtil.listCopyTo(list, NurseConfig.class);
                ExcelUtils.exportExcel(nurseConfigList, null, "sheet1", NurseConfig.class, fileName, response, 12.8f);
                break;
            case 12:
                fileName = "中医疾病" + fileName;
                List<TcmdiseaseConfig> tcmdiseaseConfigList = BeanUtil.listCopyTo(list, TcmdiseaseConfig.class);
                ExcelUtils.exportExcel(tcmdiseaseConfigList, null, "sheet1", TcmdiseaseConfig.class, fileName, response, 12.8f);
                break;
            case 13:
                fileName = "中医证候" + fileName;
                List<TcmsyndromeConfig> tcmsyndromeConfigList = BeanUtil.listCopyTo(list, TcmsyndromeConfig.class);
                ExcelUtils.exportExcel(tcmsyndromeConfigList, null, "sheet1", TcmsyndromeConfig.class, fileName, response, 12.8f);
                break;
            case 14:
                fileName = "麻醉" + fileName;
                List<AnesthesiaConfig> anesthesiaConfigList = BeanUtil.listCopyTo(list, AnesthesiaConfig.class);
                ExcelUtils.exportExcel(anesthesiaConfigList, null, "sheet1", AnesthesiaConfig.class, fileName, response, 12.8f);
                break;
            default:
                break;
        }
    }

    /**
     * 导入数据预匹配
     *
     * @param file
     * @param type
     * @param response
     */
    public void precDataMatch(MultipartFile file, Integer type, HttpServletResponse response) {
        List<MappingConfigWrapper> originList = readImportData(file, type);
        List<MappingConfigWrapper> retList = dataProcess(originList, type);
        exportExcel(response, retList, type, "关联数据（预匹配）");
    }

    /**
     * 读取导入数据
     *
     * @param file
     * @param type
     * @return
     */
    public List<MappingConfigWrapper> readImportData(MultipartFile file, Integer type) {
        List<MappingConfigWrapper> originList = Lists.newLinkedList();
        switch (type) {
            case 1:
                List<LisConfig> lisConfigList = ExcelUtils.importExcel(file, 0, 1, LisConfig.class);
                originList = BeanUtil.listCopyTo(lisConfigList, MappingConfigWrapper.class);
                break;
            case 3:
                List<PacsConfig> pacsConfigList = ExcelUtils.importExcel(file, 0, 1, PacsConfig.class);
                originList = BeanUtil.listCopyTo(pacsConfigList, MappingConfigWrapper.class);
                break;
            case 4:
                List<DiseaseConfig> diseaseConfigList = ExcelUtils.importExcel(file, 0, 1, DiseaseConfig.class);
                originList = BeanUtil.listCopyTo(diseaseConfigList, MappingConfigWrapper.class);
                break;
            case 5:
                List<DrugConfig> drugConfigList = ExcelUtils.importExcel(file, 1, 1, DrugConfig.class);
                originList = BeanUtil.listCopyTo(drugConfigList, MappingConfigWrapper.class);
                break;
            case 6:
                List<OperationConfig> operationConfigList = ExcelUtils.importExcel(file, 0, 1, OperationConfig.class);
                originList = BeanUtil.listCopyTo(operationConfigList, MappingConfigWrapper.class);
                break;
            case 7:
                List<DeptConfig> deptConfigList = ExcelUtils.importExcel(file, 0, 1, DeptConfig.class);
                originList = BeanUtil.listCopyTo(deptConfigList, MappingConfigWrapper.class);
                break;
            case 8:
                List<TransfusionConfig> transfusionConfigList = ExcelUtils.importExcel(file, 0, 1, TransfusionConfig.class);
                originList = BeanUtil.listCopyTo(transfusionConfigList, MappingConfigWrapper.class);
                break;
            case 10:
                List<ScaleConfig> scaleConfigList = ExcelUtils.importExcel(file, 0, 1, ScaleConfig.class);
                originList = BeanUtil.listCopyTo(scaleConfigList, MappingConfigWrapper.class);
                break;
            case 11:
                List<NurseConfig> nurseConfigList = ExcelUtils.importExcel(file, 0, 1, NurseConfig.class);
                originList = BeanUtil.listCopyTo(nurseConfigList, MappingConfigWrapper.class);
                break;
            case 12:
                List<TcmdiseaseConfig> tcmdiseaseConfigList = ExcelUtils.importExcel(file, 0, 1, TcmdiseaseConfig.class);
                originList = BeanUtil.listCopyTo(tcmdiseaseConfigList, MappingConfigWrapper.class);
                break;
            case 13:
                List<TcmsyndromeConfig> tcmsyndromeConfigList = ExcelUtils.importExcel(file, 0, 1, TcmsyndromeConfig.class);
                originList = BeanUtil.listCopyTo(tcmsyndromeConfigList, MappingConfigWrapper.class);
                break;
            case 14:
                List<AnesthesiaConfig> anesthesiaConfigList = ExcelUtils.importExcel(file, 0, 1, AnesthesiaConfig.class);
                originList = BeanUtil.listCopyTo(anesthesiaConfigList, MappingConfigWrapper.class);
                break;
            default:
                break;
        }
        return originList;
    }

    /**
     * 数据处理
     *
     * @param originList
     * @return
     */
    public List<MappingConfigWrapper> dataProcess(List<MappingConfigWrapper> originList, Integer type) {
        List<MappingConfigWrapper> retList = Lists.newLinkedList();
        List<MappingConfigWrapper> dataList = Lists.newLinkedList();
        List<MappingConfigWrapper> standardList = Lists.newLinkedList();
        List<MappingConfigWrapper> synonymsList = Lists.newLinkedList();
        List<MappingConfigWrapper> codeList = Lists.newLinkedList();
        List<MappingConfigWrapper> historyList = Lists.newLinkedList();

        //数据完整性校验
        for (MappingConfigWrapper item : originList) {
            //过滤医院术语名称为空的条目
            if (StringUtil.isNotBlank(item.getHisName())) {
                item.setHisName(item.getHisName().trim());
            } else {
                continue;
            }
            if (StringUtil.isNotBlank(item.getHisDetailName())) {
                item.setHisDetailName(item.getHisDetailName().trim());
            }
            if (StringUtil.isNotBlank(item.getHisCode())) {
                item.setHisCode(item.getHisCode().trim());
            }
            item.setUniqueName("");
            dataList.add(item);
        }

        if (ListUtil.isEmpty(dataList)) {
            return retList;
        }

        //标准词匹配
        List<String> nameList = dataList.stream()
                .filter(i -> StringUtil.isBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisName)
                .distinct()
                .collect(Collectors.toList());
        List<String> subNameList = dataList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisDetailName)
                .distinct()
                .collect(Collectors.toList());

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(MatchSourceEnum.StandWord.getKey());
        conceptVO.setType(type);
        conceptVO.setNames(nameList);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        conceptVO.setType(ConceptTypeEnum.Lis.getKey());
        conceptVO.setNames(subNameList);
        RespDTO<List<IndexBatchDTO>> subRespDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(subRespDTO, "标准术语校验失败");
        Map<String, List<IndexBatchDTO>> nameMap = new HashMap<>();
        Map<String, List<IndexBatchDTO>> subNameMap = new HashMap<>();
        if (ListUtil.isNotEmpty(respDTO.data)) {
            nameMap = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getName));
        }
        if (ListUtil.isNotEmpty(subRespDTO.data)) {
            subNameMap = subRespDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getName));
        }
        for (MappingConfigWrapper item : dataList) {
            List<IndexBatchDTO> indexList = Lists.newArrayList();
            if (type.equals(ConceptTypeEnum.LisPack.getKey()) && StringUtil.isNotBlank(item.getHisDetailName())) {
                if (subNameMap.containsKey(item.getHisDetailName())) {
                    indexList = subNameMap.get(item.getHisDetailName());
                }
            } else {
                if (nameMap.containsKey(item.getHisName())) {
                    indexList = nameMap.get(item.getHisName());
                }
            }

            if (ListUtil.isEmpty(indexList)) {
                standardList.add(item);
                continue;
            }
            //标准词默认取第一个
            MappingConfigWrapper standardItem = new MappingConfigWrapper();
            BeanUtil.copyProperties(item, standardItem);
            standardItem.setUniqueName(indexList.get(0).getName());
            standardItem.setConceptId(indexList.get(0).getId());
            standardItem.setCode(indexList.get(0).getCode());
            standardItem.setIsMatch(1);
            standardItem.setSource(MatchSourceEnum.StandWord.getKey());
            standardList.add(standardItem);
        }

        //同义词匹配
        List<String> synonymsNameList = standardList.stream()
                .filter(i -> (i.getIsMatch() == null || i.getIsMatch().equals(0))
                        && StringUtil.isBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisName)
                .distinct()
                .collect(Collectors.toList());
        List<String> synonymsSubNameList = standardList.stream()
                .filter(i -> (i.getIsMatch() == null || i.getIsMatch().equals(0))
                        && StringUtil.isNotBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisDetailName)
                .distinct()
                .collect(Collectors.toList());
        conceptVO.setSource(MatchSourceEnum.SynonymsWord.getKey());
        conceptVO.setType(type);
        conceptVO.setNames(synonymsNameList);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        conceptVO.setType(ConceptTypeEnum.Lis.getKey());
        conceptVO.setNames(synonymsSubNameList);
        subRespDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(subRespDTO, "标准术语校验失败");
        Map<String, List<IndexBatchDTO>> synonymsNameMap = new HashMap<>();
        Map<String, List<IndexBatchDTO>> synonymsSubNameMap = new HashMap<>();
        if (ListUtil.isNotEmpty(respDTO.data)) {
            synonymsNameMap = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getSynonyms));
        }
        if (ListUtil.isNotEmpty(subRespDTO.data)) {
            synonymsSubNameMap = subRespDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getSynonyms));
        }
        for (MappingConfigWrapper item : standardList) {
            if (item.getIsMatch() != null && item.getIsMatch().equals(1)) {
                synonymsList.add(item);
                continue;
            }
            List<IndexBatchDTO> indexList = Lists.newLinkedList();
            if (type.equals(ConceptTypeEnum.LisPack.getKey()) && StringUtil.isNotBlank(item.getHisDetailName())) {
                if (synonymsSubNameMap.containsKey(item.getHisDetailName())) {
                    indexList = synonymsSubNameMap.get(item.getHisDetailName());
                }
            } else {
                if (synonymsNameMap.containsKey(item.getHisName())) {
                    indexList = synonymsNameMap.get(item.getHisName());
                }
            }

            if (ListUtil.isEmpty(indexList)) {
                synonymsList.add(item);
                continue;
            }

            for (IndexBatchDTO index : indexList) {
                MappingConfigWrapper synonymsItem = new MappingConfigWrapper();
                BeanUtils.copyProperties(item, synonymsItem);
                synonymsItem.setUniqueName(index.getName());
                synonymsItem.setConceptId(index.getId());
                synonymsItem.setCode(index.getCode());
                synonymsItem.setIsMatch(1);
                synonymsItem.setSource(MatchSourceEnum.SynonymsWord.getKey());
                synonymsList.add(synonymsItem);
            }
        }

        //todo 编码匹配
        if (type.equals(ConceptTypeEnum.Disease.getKey())
                || type.equals(ConceptTypeEnum.Operation.getKey())
                || type.equals(ConceptTypeEnum.Tcmdisease.getKey())
                || type.equals(ConceptTypeEnum.Tcmsyndrome.getKey())) {
            List<String> codes = synonymsList.stream()
                    .filter(i -> (i.getIsMatch() == null || i.getIsMatch().equals(0))
                            && StringUtil.isNotBlank(i.getCode()))
                    .map(MappingConfigWrapper::getCode)
                    .distinct()
                    .collect(Collectors.toList());
            if (ListUtil.isEmpty(codes)) {
                codeList = BeanUtil.listCopyTo(synonymsList, MappingConfigWrapper.class);
            } else {
                conceptVO.setSource(MatchSourceEnum.Code.getKey());
                conceptVO.setType(type);
                conceptVO.setNames(codes);
                respDTO = cdssCoreClient.getConceptNames(conceptVO);
                RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
                Map<String, List<IndexBatchDTO>> codeMap = new HashMap<>();
                if (ListUtil.isNotEmpty(respDTO.data)) {
                    codeMap = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getCode));
                }
                for (MappingConfigWrapper item : synonymsList) {
                    if (item.getIsMatch() != null && item.getIsMatch().equals(1)) {
                        codeList.add(item);
                        continue;
                    }
                    if (StringUtil.isNotBlank(item.getCode()) && codeMap.containsKey(item.getCode())) {
                        List<IndexBatchDTO> indexList = codeMap.get(item.getCode());
                        for (IndexBatchDTO index : indexList) {
                            MappingConfigWrapper codeItem = new MappingConfigWrapper();
                            BeanUtils.copyProperties(item, codeItem);
                            codeItem.setUniqueName(index.getName());
                            codeItem.setConceptId(index.getId());
                            codeItem.setCode(index.getCode());
                            codeItem.setIsMatch(1);
                            codeItem.setSource(MatchSourceEnum.Code.getKey());
                            codeList.add(codeItem);
                        }
                    } else {
                        codeList.add(item);
                        continue;
                    }
                }
            }
        } else {
            codeList = BeanUtil.listCopyTo(synonymsList, MappingConfigWrapper.class);
        }

        //todo 历史匹配
        List<String> hisNames = codeList.stream()
                .filter(i -> i.getIsMatch() == null || i.getIsMatch().equals(0))
                .map(MappingConfigWrapper::getHisName)
                .distinct()
                .collect(Collectors.toList());
        if (ListUtil.isNotEmpty(hisNames)) {
            Map<String, Map<String, Map<String, List<String>>>> configMap = groupByHisNameWithName(hisNames, type, null);

            for (MappingConfigWrapper item : codeList) {
                if (item.getIsMatch() != null && item.getIsMatch().equals(1)) {
                    retList.add(item);
                }
                if (configMap.containsKey(item.getHisName())) {
                    Map<String, Map<String, List<String>>> subMap = configMap.get(item.getHisName());
                    for (Map.Entry<String, Map<String, List<String>>> subEntry : subMap.entrySet()) {
                        for (String name : subEntry.getValue().keySet()) {
                            MappingConfigWrapper historyItem = new MappingConfigWrapper();
                            BeanUtils.copyProperties(item, historyItem);
                            historyItem.setHisDetailName(subEntry.getKey());
                            historyItem.setUniqueName(name);
                            historyItem.setIsMatch(1);
                            historyItem.setSource(MatchSourceEnum.History.getKey());
                            if (type.equals(ConceptTypeEnum.Drug.getKey())) {
                                List<String> forms = subEntry.getValue().get(name);
                                if (ListUtil.isEmpty(forms)) {
                                    retList.add(historyItem);
                                } else {
                                    for (String form : forms) {
                                        MappingConfigWrapper formItem = new MappingConfigWrapper();
                                        BeanUtil.copyProperties(historyItem, formItem);
                                        formItem.setForm(form);
                                        retList.add(formItem);
                                    }
                                }
                            } else {
                                retList.add(historyItem);
                            }
                        }
                    }
                }
            }
        } else {
            retList = BeanUtil.listCopyTo(codeList, MappingConfigWrapper.class);
        }
        return retList;
    }

    /**
     * 根据医院名称分组-返回id
     *
     * @param hisNames
     * @param type
     * @return
     */
    public Map<String, Map<String, Map<Long, List<Long>>>> groupByHisNameWithId(List<String> hisNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<Long, List<Long>>>> retMap = new HashMap<>();
        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("is_match", 1)
                .eq("type", type);
        if (hospitalId != null) {
            queryWrapper.eq("hospital_id", hospitalId);
        }
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        List<MappingConfig> list = this.list(queryWrapper);

        if (ListUtil.isEmpty(list)) {
            return retMap;
        }

        for (MappingConfig item : list) {
            if (StringUtil.isBlank(item.getHisDetailName())) {
                item.setHisDetailName("");
            }
        }

        Map<String, List<MappingConfig>> hisMap = list.stream().collect(Collectors.groupingBy(MappingConfig::getHisName));
        for (Map.Entry<String, List<MappingConfig>> entry : hisMap.entrySet()) {
            Map<String, Map<Long, List<Long>>> subMap = new HashMap<>();
            Map<String, List<MappingConfig>> detailMap = entry.getValue().stream()
                    .collect(Collectors.groupingBy(MappingConfig::getHisDetailName));
            for (Map.Entry<String, List<MappingConfig>> subEntry : detailMap.entrySet()) {
                subMap.put(subEntry.getKey(), subEntry.getValue().stream().collect(Collectors.groupingBy(MappingConfig::getConceptId,
                        Collectors.mapping(MappingConfig::getFormConceptId, Collectors.toList()))));
            }
            retMap.put(entry.getKey(), subMap);
        }
        return retMap;
    }

    /**
     * 根据医院名称分组-返回名称
     *
     * @param hisNames
     * @param type
     * @param hospitalId
     * @return
     */
    public Map<String, Map<String, Map<String, List<String>>>> groupByHisNameWithName(List<String> hisNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<String, List<String>>>> retMap = new HashMap<>();
        Map<String, Map<String, Map<Long, List<Long>>>> idMap = groupByHisNameWithId(hisNames, type, hospitalId);

        //conceptIds
        List<Long> ids = idMap.values()
                .stream()
                .flatMap(i -> i.values().stream())
                .map(i -> i.keySet())
                .flatMap(Collection::stream)
                .filter(i -> i != null)
                .distinct()
                .collect(Collectors.toList());

        //formConceptId
        ids.addAll(idMap.values()
                .stream()
                .flatMap(i -> i.values().stream())
                .flatMap(i -> i.values().stream())
                .flatMap(Collection::stream)
                .filter(i -> i != null)
                .distinct()
                .collect(Collectors.toList()));

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(-1);
        conceptVO.setType(type);
        conceptVO.setIds(ids);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<IndexBatchDTO> indexList = respDTO.data;
        if (ListUtil.isNotEmpty(indexList)) {
            Map<Long, List<IndexBatchDTO>> indexMap
                    = indexList.stream().collect(Collectors.groupingBy(IndexBatchDTO::getId));
            for (Map.Entry<String, Map<String, Map<Long, List<Long>>>> entry : idMap.entrySet()) {
                Map<String, Map<String, List<String>>> subMap = new HashMap<>();
                for (Map.Entry<String, Map<Long, List<Long>>> subEntry : entry.getValue().entrySet()) {
                    Map<String, List<String>> thirdMap = new HashMap<>();
                    List<String> uniqueNames = Lists.newArrayList();
                    List<String> forms = Lists.newArrayList();
                    for (Long conceptId : subEntry.getValue().keySet()) {
                        uniqueNames.addAll(indexMap.get(conceptId).stream().map(IndexBatchDTO::getName).collect(Collectors.toList()));
                    }
                    List<Long> formConceptIds = subEntry.getValue().values()
                            .stream()
                            .flatMap(Collection::stream)
                            .distinct()
                            .collect(Collectors.toList());
                    for (Long formConceptId : formConceptIds) {
                        if (formConceptId == null) {
                            forms.add("");
                        } else {
                            forms.addAll(indexMap.get(formConceptId).stream().map(IndexBatchDTO::getName).collect(Collectors.toList()));
                        }
                    }
                    forms = forms.stream().distinct().collect(Collectors.toList());
                    for (String uniqueName : uniqueNames) {
                        thirdMap.put(uniqueName, forms);
                    }

                    subMap.put(subEntry.getKey(), thirdMap);
                }
                retMap.put(entry.getKey(), subMap);
            }
        }
        return retMap;
    }
}