package com.diagbot.facade;

import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.DiseaseInfoDTO;
import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.NurseInfoDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalConceptDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.ScaleInfoDTO;
import com.diagbot.dto.TcmdiseaseInfoDTO;
import com.diagbot.dto.TcmsyndromeInfoDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.HospitalCheckVO;
import com.diagbot.vo.RetrievalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:44
 */
@Component
public class RetrievalFacade {
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;
    @Autowired
    private LisConfigFacade lisConfigFacade;
    @Autowired
    private PacsConfigFacade pacsConfigFacade;
    @Autowired
    private DiseaseConfigFacade diseaseConfigFacade;
    @Autowired
    private DrugConfigFacade drugConfigFacade;
    @Autowired
    private OperationConfigFacade operationConfigFacade;
    @Autowired
    private DeptConfigFacade deptConfigFacade;
    @Autowired
    private TransfusionConfigFacade transfusionConfigFacade;
    @Autowired
    private ScaleConfigFacade scaleConfigFacade;
    @Autowired
    private NurseConfigFacade nurseConfigFacade;
    @Autowired
    private TcmdiseaseConfigFacade tcmdiseaseConfigFacade;
    @Autowired
    private TcmsyndromeConfigFacade tcmsyndromeConfigFacade;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        RetrievalDTO retrievalDTO = new RetrievalDTO();
        if (null == retrievalVO
                || (null != retrievalVO
                && (null == retrievalVO.getHospitalId() || retrievalVO.getHospitalId().equals(-1L)))) {
            //无医院或者朗通医院查询标准词
            RespDTO<RetrievalDTO> respDTO = cdssCoreClient.index(retrievalVO);
            RespDTOUtil.respNGDealCover(respDTO, "检索失败");
            retrievalDTO = respDTO.data;
        } else {
            //实际医院查询医院词
            if (StringUtil.isBlank(retrievalVO.getInputStr())) {
                throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入检索内容！");
            }
            HospitalCheckVO hospitalCheckVO = new HospitalCheckVO();
            hospitalCheckVO.setHospitalId(retrievalVO.getHospitalId());
            if (!hospitalInfoFacade.checkHospital(hospitalCheckVO)) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "该医院ID不存在或者未启用！");
            } else {
                switch (retrievalVO.getType()) {
                    case 1:
                        List<RetrievalConceptDTO> lisNames = new ArrayList<>();
                        List<String> lisPacks = lisConfigFacade.getLisPacks(retrievalVO);
                        for (String str : lisPacks) {
                            RetrievalConceptDTO data = new RetrievalConceptDTO();
                            data.setName(str);
                            lisNames.add(data);
                        }
                        retrievalDTO.setNameList(lisNames);
                        break;
                    case 2:
                        retrievalDTO.setLisDetailNames(lisConfigFacade.getLis(retrievalVO));
                        break;
                    case 3:
                        List<RetrievalConceptDTO> Pacss = new ArrayList<>();
                        List<String> Pacsslist = pacsConfigFacade.getPACSs(retrievalVO);
                        for (String str : Pacsslist) {
                            RetrievalConceptDTO data = new RetrievalConceptDTO();
                            data.setName(str);
                            Pacss.add(data);
                        }
                        retrievalDTO.setNameList(Pacss);
                        break;
                    case 4:
                        List<DiseaseInfoDTO> diseases = diseaseConfigFacade.getDiseases(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(diseases, RetrievalConceptDTO.class));
                        break;
                    case 5:
                        List<DrugInfoDTO> medicines = drugConfigFacade.getMedicines(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(medicines, RetrievalConceptDTO.class));
                        break;
                    case 6:
                        List<OperationInfoDTO> operations = operationConfigFacade.getOperations(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(operations, RetrievalConceptDTO.class));
                        break;
                    case 7:
                        List<RetrievalConceptDTO> Depts = new ArrayList<>();
                        List<String> Deptslist = deptConfigFacade.getDepts(retrievalVO);
                        for (String str : Deptslist) {
                            RetrievalConceptDTO data = new RetrievalConceptDTO();
                            data.setName(str);
                            Depts.add(data);
                        }
                        retrievalDTO.setNameList(Depts);
                        break;
                    case 8:
                        List<RetrievalConceptDTO> Transfusion = new ArrayList<>();
                        List<String> Transfusions = transfusionConfigFacade.getTransfusion(retrievalVO);
                        for (String str : Transfusions) {
                            RetrievalConceptDTO data = new RetrievalConceptDTO();
                            data.setName(str);
                            Transfusion.add(data);
                        }
                        retrievalDTO.setNameList(Transfusion);
                        break;
                    case 10:
                        List<ScaleInfoDTO> scales = scaleConfigFacade.getScales(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(scales, RetrievalConceptDTO.class));
                        break;
                    case 11:
                        List<NurseInfoDTO> nurses = nurseConfigFacade.getNurses(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(nurses, RetrievalConceptDTO.class));
                        break;
                    case 12:
                        List<TcmdiseaseInfoDTO> tcmdiseases = tcmdiseaseConfigFacade.getTcmdiseases(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(tcmdiseases, RetrievalConceptDTO.class));
                        break;
                    case 13:
                        List<TcmsyndromeInfoDTO> tcmsyndromes = tcmsyndromeConfigFacade.getTcmsyndromes(retrievalVO);
                        retrievalDTO.setNameList(BeanUtil.listCopyTo(tcmsyndromes, RetrievalConceptDTO.class));
                        break;
                }
            }
        }
        return retrievalDTO;
    }
}